#!/usr/bin/python

# Copyright (C) 2005 Thomas Petazzoni <thomas.petazzoni@enix.org>
# Copyright (C) 2005 Christophe Lucas <clucas@rotomalug.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

cookiefile = ".mypixmania.cookie"

authors = [ "Thomas Petazzoni <thomas.petazzoni@enix.org>", "Christophe Lucas <clucas@rotomalug.org>" ]
version = "0.1.4"

import sys
import pycurl
import StringIO
import re
import os
import ConfigParser
import posix

class PixMania:

    __curl_instancied = 0

    #
    # Get a valid PHP session ID to login on the Pixmania website
    #
    # Returns:
    #  A string containing a valid PHP session ID
    #
    def findPhpSessId(self):
	curl = self.__curl
	# Get the Pixmania homepage
	contents = StringIO.StringIO()
	curl.setopt(curl.URL, 'http://www.mypixmania.com')
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

	# Extract the PHP Session ID
	match = re.search(r".*PHPSESSID=([0-9a-f]+).*",
                      contents.getvalue())

	if match is None:
        	print "Error: cannot get PHPSESSID from site"
		return

	return match.group(1)

    #
    # Login on the Mypixmania website, using the given PHP session ID,
    # login and password. This function will create a cookie file which
    # will be used in further requests
    #
    # Returns:
    #  False on success
    #  True if login failed
    #
    def loginOn(self):
	contents = StringIO.StringIO()
	curl = self.__curl
	# Issue a POST request with the login and password
	curl.setopt(curl.URL,
                'http://www.mypixmania.com/fr/fr/home/?PHPSESSID=' + self.__phpsessid)
	curl.setopt(curl.COOKIEJAR, cookiefile)
	curl.setopt(curl.HTTPPOST,
                [('prov', ''),
                 ('verification_login','1'),
                 ('login', self.__login),
                 ('pass', self.__password)])
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

	# Check that login was successful
	match = re.search(r".*Bienvenue.*</td>", contents.getvalue())
	if match is None:
        	return False
	else:
        	return True

    #
    # Get the list of albums
    #
    # Returns:
    #  A dictionnary which contains the album ID's and names. ID's are
    #  used as keys for the dictionnary, since they are unique.
    #
    def getAlbums(self):
    	contents = StringIO.StringIO()
	curl = self.__curl
	curl.setopt(curl.URL,
                'http://www.mypixmania.com/fr/fr/album-photo/')
	curl.setopt(curl.POST, False)
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

	names = re.findall(r'.*addAlbum\(\n\s.*,\n\s"(.*)",\n.*',
                     contents.getvalue())
	ids = re.findall(r'.*addAlbum\(\n\s(.*),\n.*',
                      contents.getvalue())

	return dict(zip(ids, names))

    #
    # Delete an album
    #
    # Params:
    #  album_id: The ID of the album
    #
    # Returns:
    #  Nothing
    #
    def deleteAlbum(self, album_id):
	contents = StringIO.StringIO()
	curl = self.__curl
	curl.setopt(curl.URL,
	 	'http://www.mypixmania.com/fr/fr/album-photo/supprimer?current_album=' + album_id
	)
	curl.setopt(curl.POST, False)
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

    #
    # Find the base directory for a given album.
    #
    # Params:
    #  album_id: The ID of the album
    #
    # Returns:
    #  A string containing the base directory for the album
    #
    def getBaseDirectory(self, album_id):
	contents = StringIO.StringIO()
	curl = self.__curl
	curl.setopt(curl.URL,
		'http://www.mypixmania.com/fr/fr/album-photo/commander?com=&current_album=' + album_id
	)
	curl.setopt(curl.POST, False)
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

	dir = re.findall(r'directory.=."([a-zA-Z0-9_\/]+)";', contents.getvalue())
	return dir[0]

    #
    # Find the number of pictures in an album
    #
    # Params:
    #  album_id: The ID of the album
    #
    # Returns:
    #  An integer containing the number of pictures of the album
    #
    def getNrPictureInAlbum(self, album_id):
	return len(self.getThumbnailsId(album_id))

    #
    # Get the picture thumbnails ids and names for a given album
    #
    # Params:
    #  album_id: The ID of the album
    #
    # Returns:
    #
    #  A dictionary which associates thumbnails IDs to thumbnails names.
    #
    def getThumbnailsId(self, album_id):
	contents = StringIO.StringIO()
	curl = self.__curl
	curl.setopt(curl.URL,
		'http://www.mypixmania.com/fr/fr/album-photo/commander?com=&current_album=' + album_id
	)
	curl.setopt(curl.POST, False)
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.WRITEFUNCTION, contents.write)
	curl.perform()

	names = re.findall(r'.*addPhoto\(\n\s.*,\n\s"(.*)",\n.*', contents.getvalue())
	ids = re.findall(r'.*addPhoto\(\n\s.*,\n\s.*,\n\s.*,\n\s.*,\n\s.*\n\s"(.*)"\n.*', contents.getvalue())

	return dict(zip(ids, names))

    #
    # Download the thumbnail for a given picture
    #
    # Params:
    #  album_id:        The ID of the album
    #  id:              ID of the thumbnail
    #
    # Caller must make sure that the path given as a filename is
    # inside an already existing directory.
    #
    # Returns:
    #  Contents of the picture.
    #
    def getThumbnailPictureFor(self, album_id, id):

        contents = StringIO.StringIO()
# From clucas
#        url = self.getBaseDirectory(album_id) + "/" + id
        directory = self.getBaseDirectory(album_id) 
	url = 'http://www.mypixmania.com/tools/thumbnail.php?d=' + directory + '&src0=id(' + id + ')'
        print "Fetching from URL " + url
        curl = self.__curl

        curl.setopt(curl.URL,
			url
# From clucas
#                    'http://www.mypixmania.com/T/'  + url
                    )
        curl.setopt(curl.POST, False)
        curl.setopt(curl.COOKIEFILE, cookiefile)
        curl.setopt(curl.WRITEFUNCTION, contents.write)
        curl.perform()

        print "HTTP result code: " + str(curl.getinfo(curl.HTTP_CODE))

        return contents.getvalue()

    #
    # Get the a full-sized picture
    #
    # Params:
    #  album_id:       ID of the album
    #  id:             ID of the picture
    #
    # Returns:
    #  Contents of the picture.
    #
    def getPictureFor(self, album_id, id):

        contents = StringIO.StringIO()
        url = self.getBaseDirectory(album_id) + "/" + id
        curl = self.__curl
        curl.setopt(curl.URL,
                    'http://www.mypixmania.com/S/'  + url
                    )
        curl.setopt(curl.POST, False)
        curl.setopt(curl.COOKIEFILE, cookiefile)
        curl.setopt(curl.WRITEFUNCTION, contents.write)
        curl.perform()

        return contents.getvalue()

    #
    # Upload a picture to an album
    #
    # Params:
    #  album           : The ID of the album in which the picture should
    #                    be uploaded
    #  picturefilename : Filename of the picture to
    #  uploadCallback  : A callback which is called to display
    #                    information about the current picture
    #                    being uploaded. The prototype of the
    #                    callback should be callback(download_t,
    #                    download_d, upload_t, upload_d) where
    #                    download_t is the total number of bytes
    #                    to download, download_d, the number of
    #                    bytes already downloaded, upload_t the
    #                    total number of bytes to upload, and
    #                    upload_d the number of bytes already
    #                    uploaded. (Optional)
    #
    # Returns:
    #  Nothing
    #
    def addPicture(self, album, picturefilename, uploadCallback = None):
    	result = StringIO.StringIO()

	curl = self.__curl

	# Issue the POST request
	curl.setopt(curl.URL,
                'http://www.mypixmania.com/fr/fr/vision/index.php?fuseaction=update_album&rajouterphoto=1')
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.HTTPPOST,
                [('current_album',album),
                 ('album_name',''),
                 ('album_theme',''),
                 ('nbre_photo','5'),
                 ('from','importer'),
                 ('userfile1',(pycurl.FORM_FILE, picturefilename))])
	curl.setopt(curl.WRITEFUNCTION,
	            result.write)
	curl.setopt(curl.NOPROGRESS, False)

        if uploadCallback is not None:
            curl.setopt(curl.PROGRESSFUNCTION, uploadCallback)

	curl.perform()

	# Finalize callback display
	print ""

    #
    # Create a new album
    #
    # Params:
    #  albumname:       Name of the new album
    #  filelist:        List of files to add to the new album
    #  uploadCallback:  A callback which is called to display
    #                   information about the current picture
    #                   being uploaded. The prototype of the
    #                   callback should be callback(download_t,
    #                   download_d, upload_t, upload_d) where
    #                   download_t is the total number of bytes
    #                   to download, download_d, the number of
    #                   bytes already downloaded, upload_t the
    #                   total number of bytes to upload, and
    #                   upload_d the number of bytes already
    #                   uploaded. (Optional)
    # progressCallback: A callback which is called to display which
    #                   picture is being uploaded. Its prototype is
    #                   callback(pictureid, picturenb, albumname,
    #                   filename), where pictureid is the number
    #                   of the current picture being uploaded,
    #                   picturenb the number of pictures to upload,
    #                   albumname the name of the destination album,
    #                   and filename the name of the file being
    #                   uploaded. (Optional)
    #
    # Returns:
    #  Nothing
    #
    def createAlbum(self, albumname, filelist, uploadCallback = None, progressCallback = None):
    	result = StringIO.StringIO()

	curl = self.__curl

	picturenb = len(filelist)
	pictureid = 1

	# Issue the POST request
	curl.setopt(curl.URL,
                'http://www.mypixmania.com/fr/fr/vision/index.php?fuseaction=creation_album&flag=creeralbum')
	curl.setopt(curl.COOKIEFILE, cookiefile)
	curl.setopt(curl.HTTPPOST,
                [('album_name',albumname),
                 ('album_theme', ''),
                 ('nbre_photo', '30'),
                 ('select','1'),
                 ('from', 'importer'),
                 ('userfile1',(pycurl.FORM_FILE, filelist[0]))])
	curl.setopt(curl.WRITEFUNCTION, result.write)
	curl.setopt(curl.NOPROGRESS, False)

        if uploadCallback is not None:
            curl.setopt(curl.PROGRESSFUNCTION, uploadCallback)
        if progressCallback is not None:
                progressCallback(pictureid, picturenb, albumname,  filelist[0])


	curl.perform()
	if curl.getinfo(curl.HTTP_CODE) != 200:
        	print "Error while creating album"
		sys.exit (1)

	pictureid = pictureid + 1

	# Finalize callback display
	print ""

	for f in filelist[1:]:
            if progressCallback is not None:
                progressCallback(pictureid, picturenb, albumname, f)
            self.addPicture(albumname, f, uploadCallback)
            pictureid = pictureid + 1

    #
    # Displays some information about the syntax of the configuration file
    #
    # Returns:
    #  Nothing
    #
    def configFileUsage(self):
    	print "Please create one as follows:"
	print "[account]"
	print "login = your.pixmania.login@something.org"
	print "password = your_password"

    #
    # Get configuration values from the configuration file
    #
    # Returns
    #  A list containing the login and the password
    #
    def getConfig(self):
    	config = ConfigParser.ConfigParser()

	try:
        	config.readfp(open(posix.environ['HOME'] + '/.mypixmania.cfg'))
	except IOError:
        	print "Couldn't open configuration file ~/.mypixmania.cfg"
		self.configFileUsage()
		sys.exit(1)

	try:
        	self.__login = config.get('account', 'login')
		self.__password = config.get('account', 'password')
	except ConfigParser.NoOptionError:
        	print "Couldn't find 'login' or 'password' in configuration file"
		configFileUsage()
		sys.exit(1)

	return [ self.__login, self.__password ]

    #
    # Constructor
    #
    def __init__(self):
	ret = self.getConfig()
	if ret == None:
		return 1
	else:
		self.__curl_instancied = 1
	self.__curl = pycurl.Curl()
	self.__phpsessid = self.findPhpSessId()
	self.loginOn()

    #
    # Destructor
    #
    def __del__(self):
    	if self.__curl_instancied == 1:
	        self.__curl.close()
        if (os.path.exists (cookiefile)):
            os.remove (cookiefile)
